<?php
/**
 * MovimentoList Listing
 * @author  <your name here>
 */
class MovimentoList extends TPage
{
    private $form; // form
    private $datagrid; // listing
    private $pageNavigation;
    private $formgrid;
    private $loaded;
    private $deleteButton;
    
    /**
     * Class constructor
     * Creates the page, the form and the listing
     */
    public function __construct()
    {
        parent::__construct();
        
        // creates the form
        $this->form = new BootstrapFormBuilder('form_search_Movimento');
        $this->form->setFormTitle('Movimento');
        

        // create the form fields
        $cadtipooperacao_id = new TDBCombo('cadtipooperacao_id', 'goldnotas', 'Cadtipooperacao', 'id', 'descricao');
        $numero = new TEntry('numero');
        $serie = new TEntry('serie');
        $situacao = new TCombo('situacao');
        $cadcondpgto_id = new TDBCombo('cadcondpgto_id', 'goldnotas', 'Cadcondpgto', 'id', 'descricao');
        
        $filtro = new TCriteria;                     
        $filtro->add(new TFilter('ecliente', '=', "S"), TExpression::OR_OPERATOR);
        $filtro->add(new TFilter('efornecedor', '=', "S"), TExpression::OR_OPERATOR);         
        
        $cadpessoa_id = new TDBCombo('cadpessoa_id', 'goldnotas', 'Cadpessoa', 'id', 'pessoa', null, $filtro, false, false);
        $dataemissao1 = new TDate('dataemissao1');
        $dataemissao2 = new TDate('dataemissao2');

        
        $itens = array();
        $itens['1'] = 'Pendente';
        $itens['2'] = 'Cancelado';
        $itens['3'] = 'Faturado';     
        $situacao->addItems($itens);
        
        $dataemissao1->setMask('dd/mm/yyyy');
        $dataemissao1->setDatabaseMask('yyyy-mm-dd');
        $dataemissao2->setMask('dd/mm/yyyy');
        $dataemissao2->setDatabaseMask('yyyy-mm-dd');                
        
        $cadcondpgto_id->enableSearch();
        $cadpessoa_id->enableSearch();
        $cadtipooperacao_id->enableSearch();
        
        // add the fields
        $row = $this->form->addFields( [ new TLabel('Tipo de Operação') , $cadtipooperacao_id ],
                                       [ new TLabel('Número') , $numero ],
                                       [ new TLabel('Série') ,  $serie ],
                                       [ new TLabel('Situação') ,  $situacao ] );
        $row->layout = ['col-sm-4 control-label',
                        'col-sm-2 control-label',
                        'col-sm-2 control-label',
                        'col-sm-2 control-label'];                                   
                              
        $row = $this->form->addFields( [ new TLabel('Cliente/Fornecedor') , $cadpessoa_id ],
                                       [ new TLabel('Condição de Pagamento') , $cadcondpgto_id ]);
        $row->layout = ['col-sm-5 control-label',
                        'col-sm-5 control-label'];                                       
        $row = $this->form->addFields( [ new TLabel('Dt. Emissão Inicial') , $dataemissao1 ],
                                       [ new TLabel('Dt. Emissão Final') ,  $dataemissao2 ] );
        $row->layout = ['col-sm-2 control-label',
                        'col-sm-2 control-label'];

        // set sizes
        $cadtipooperacao_id->setSize('100%');
        $numero->setSize('100%');
        $serie->setSize('100%');
        $situacao->setSize('100%');
        $cadcondpgto_id->setSize('100%');
        $cadpessoa_id->setSize('100%');
        $dataemissao1->setSize('100%');
        $dataemissao2->setSize('100%');

        
        // keep the form filled during navigation with session data
        $this->form->setData( TSession::getValue(__CLASS__ . '_filter_data') );
        
        // add the search form actions
        $btn = $this->form->addAction(_t('Find'), new TAction([$this, 'onSearch']), 'fa:search');
        $btn->class = 'btn btn-sm btn-primary';
        $this->form->addActionLink(_t('New'), new TAction(['MovimentoForm', 'onEdit']), 'fa:plus green');
        
        // creates a Datagrid
        $this->datagrid = new BootstrapDatagridWrapper(new TDataGrid);
        $this->datagrid->style = 'width: 100%';
        $this->datagrid->datatable = 'true';
        // $this->datagrid->enablePopover('Popover', 'Hi <b> {name} </b>');
        

        // creates the datagrid columns
        $column_id = new TDataGridColumn('id', 'Id', 'right');
        $column_cadtipooperacao_id = new TDataGridColumn('cadtipooperacao_id', 'Cadtipooperacao Id', 'right');
        $column_numero = new TDataGridColumn('numero', 'Numero', 'left');
        $column_serie = new TDataGridColumn('serie', 'Serie', 'left');
        $column_situacao = new TDataGridColumn('situacao', 'Situacao', 'right');
        $column_cadcondpgto_id = new TDataGridColumn('cadcondpgto_id', 'Cadcondpgto Id', 'right');
        $column_cadpessoa_id = new TDataGridColumn('cadpessoa_id', 'Cadpessoa Id', 'right');
        $column_dataemissao = new TDataGridColumn('dataemissao', 'Dataemissao', 'left');
        $column_subtotal = new TDataGridColumn('subtotal', 'Subtotal', 'left');
        $column_desconto = new TDataGridColumn('desconto', 'Desconto', 'left');
        $column_total = new TDataGridColumn('total', 'Total', 'left');


        // add the columns to the DataGrid
        $this->datagrid->addColumn($column_id);
        $this->datagrid->addColumn($column_cadtipooperacao_id);
        $this->datagrid->addColumn($column_numero);
        $this->datagrid->addColumn($column_serie);
        $this->datagrid->addColumn($column_situacao);
        $this->datagrid->addColumn($column_cadcondpgto_id);
        $this->datagrid->addColumn($column_cadpessoa_id);
        $this->datagrid->addColumn($column_dataemissao);
        $this->datagrid->addColumn($column_subtotal);
        $this->datagrid->addColumn($column_desconto);
        $this->datagrid->addColumn($column_total);


        $action1 = new TDataGridAction(['MovimentoForm', 'onEdit'], ['id'=>'{id}']);
        $action2 = new TDataGridAction([$this, 'onDelete'], ['id'=>'{id}']);
        
        //$this->datagrid->addAction($action1, _t('Edit'),   'far:edit blue');
        $this->datagrid->addAction($action2 ,_t('Delete'), 'far:trash-alt red');
        
        //create the datagrid model
        $this->datagrid->createModel();
        
        // creates the page navigation
        $this->pageNavigation = new TPageNavigation;
        $this->pageNavigation->setAction(new TAction([$this, 'onReload']));
        $this->pageNavigation->setWidth($this->datagrid->getWidth());
        
        // vertical box container
        $container = new TVBox;
        $container->style = 'width: 100%';
        // $container->add(new TXMLBreadCrumb('menu.xml', __CLASS__));
        $container->add($this->form);
        $container->add(TPanelGroup::pack('', $this->datagrid, $this->pageNavigation));
        
        parent::add($container);
    }
    
    /**
     * Inline record editing
     * @param $param Array containing:
     *              key: object ID value
     *              field name: object attribute to be updated
     *              value: new attribute content 
     */
    public function onInlineEdit($param)
    {
        try
        {
            // get the parameter $key
            $field = $param['field'];
            $key   = $param['key'];
            $value = $param['value'];
            
            TTransaction::open('goldnotas'); // open a transaction with database
            $object = new Movimento($key); // instantiates the Active Record
            $object->{$field} = $value;
            $object->store(); // update the object in the database
            TTransaction::close(); // close the transaction
            
            $this->onReload($param); // reload the listing
            new TMessage('info', "Record Updated");
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    
    /**
     * Register the filter in the session
     */
    public function onSearch()
    {
        // get the search form data
        $data = $this->form->getData();
        
        // clear session filters
        TSession::setValue(__CLASS__.'_filter_cadtipooperacao_id',   NULL);
        TSession::setValue(__CLASS__.'_filter_numero',   NULL);
        TSession::setValue(__CLASS__.'_filter_serie',   NULL);
        TSession::setValue(__CLASS__.'_filter_situacao',   NULL);
        TSession::setValue(__CLASS__.'_filter_cadcondpgto_id',   NULL);
        TSession::setValue(__CLASS__.'_filter_cadpessoa_id',   NULL);
        TSession::setValue(__CLASS__.'_filter_dataemissao1',   NULL);
        TSession::setValue(__CLASS__.'_filter_dataemissao2',   NULL);

        if (isset($data->cadtipooperacao_id) AND ($data->cadtipooperacao_id)) {
            $filter = new TFilter('cadtipooperacao_id', '=', $data->cadtipooperacao_id); // create the filter
            TSession::setValue(__CLASS__.'_filter_cadtipooperacao_id',   $filter); // stores the filter in the session
        }


        if (isset($data->numero) AND ($data->numero)) {
            $filter = new TFilter('numero', 'like', "%{$data->numero}%"); // create the filter
            TSession::setValue(__CLASS__.'_filter_numero',   $filter); // stores the filter in the session
        }


        if (isset($data->serie) AND ($data->serie)) {
            $filter = new TFilter('serie', 'like', "%{$data->serie}%"); // create the filter
            TSession::setValue(__CLASS__.'_filter_serie',   $filter); // stores the filter in the session
        }


        if (isset($data->situacao) AND ($data->situacao)) {
            $filter = new TFilter('situacao', 'like', "%{$data->situacao}%"); // create the filter
            TSession::setValue(__CLASS__.'_filter_situacao',   $filter); // stores the filter in the session
        }


        if (isset($data->cadcondpgto_id) AND ($data->cadcondpgto_id)) {
            $filter = new TFilter('cadcondpgto_id', '=', $data->cadcondpgto_id); // create the filter
            TSession::setValue(__CLASS__.'_filter_cadcondpgto_id',   $filter); // stores the filter in the session
        }


        if (isset($data->cadpessoa_id) AND ($data->cadpessoa_id)) {
            $filter = new TFilter('cadpessoa_id', '=', $data->cadpessoa_id); // create the filter
            TSession::setValue(__CLASS__.'_filter_cadpessoa_id',   $filter); // stores the filter in the session
        }


        if (isset($data->dataemissao1) AND ($data->dataemissao1)) {
            $filter = new TFilter('dataemissao', '>=', $data->dataemissao1); // create the filter
            TSession::setValue(__CLASS__.'_filter_dataemissao1',   $filter); // stores the filter in the session
        }


        if (isset($data->dataemissao2) AND ($data->dataemissao2)) {
            $filter = new TFilter('dataemissao', '<=', $data->dataemissao2); // create the filter
            TSession::setValue(__CLASS__.'_filter_dataemissao2',   $filter); // stores the filter in the session
        }

        
        // fill the form with data again
        $this->form->setData($data);
        
        // keep the search data in the session
        TSession::setValue(__CLASS__ . '_filter_data', $data);
        
        $param = array();
        $param['offset']    =0;
        $param['first_page']=1;
        $this->onReload($param);
    }
    
    /**
     * Load the datagrid with data
     */
    public function onReload($param = NULL)
    {
        try
        {
            // open a transaction with database 'goldnotas'
            TTransaction::open('goldnotas');
            
            // creates a repository for Movimento
            $repository = new TRepository('Movimento');
            $limit = 10;
            // creates a criteria
            $criteria = new TCriteria;
            
            // default order
            if (empty($param['order']))
            {
                $param['order'] = 'id';
                $param['direction'] = 'asc';
            }
            $criteria->setProperties($param); // order, offset
            $criteria->setProperty('limit', $limit);
            

            if (TSession::getValue(__CLASS__.'_filter_cadtipooperacao_id')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_cadtipooperacao_id')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_numero')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_numero')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_serie')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_serie')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_situacao')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_situacao')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_cadcondpgto_id')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_cadcondpgto_id')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_cadpessoa_id')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_cadpessoa_id')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_dataemissao1')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_dataemissao1')); // add the session filter
            }


            if (TSession::getValue(__CLASS__.'_filter_dataemissao2')) {
                $criteria->add(TSession::getValue(__CLASS__.'_filter_dataemissao2')); // add the session filter
            }

            
            // load the objects according to criteria
            $objects = $repository->load($criteria, FALSE);
            
            if (is_callable($this->transformCallback))
            {
                call_user_func($this->transformCallback, $objects, $param);
            }
            
            $this->datagrid->clear();
            if ($objects)
            {
                // iterate the collection of active records
                foreach ($objects as $object)
                {
                    // add the object inside the datagrid
                    $this->datagrid->addItem($object);
                }
            }
            
            // reset the criteria for record count
            $criteria->resetProperties();
            $count= $repository->count($criteria);
            
            $this->pageNavigation->setCount($count); // count of records
            $this->pageNavigation->setProperties($param); // order, page
            $this->pageNavigation->setLimit($limit); // limit
            
            // close the transaction
            TTransaction::close();
            $this->loaded = true;
        }
        catch (Exception $e)
        {
            new TMessage('error', $e->getMessage());
            TTransaction::rollback();
        }
    }
    
    /**
     * Ask before deletion
     */
    public static function onDelete($param)
    {
        // define the delete action
        $action = new TAction([__CLASS__, 'Delete']);
        $action->setParameters($param); // pass the key parameter ahead
        
        // shows a dialog to the user
        new TQuestion(AdiantiCoreTranslator::translate('Do you really want to delete ?'), $action);
    }
    
    /**
     * Delete a record
     */
    public static function Delete($param)
    {
        try
        {
            $key=$param['key']; // get the parameter $key
            TTransaction::open('goldnotas'); // open a transaction with database
            $object = new Movimento($key, FALSE); // instantiates the Active Record
            $object->delete(); // deletes the object from the database
            TTransaction::close(); // close the transaction
            
            $pos_action = new TAction([__CLASS__, 'onReload']);
            new TMessage('info', AdiantiCoreTranslator::translate('Record deleted'), $pos_action); // success message
        }
        catch (Exception $e) // in case of exception
        {
            new TMessage('error', $e->getMessage()); // shows the exception error message
            TTransaction::rollback(); // undo all pending operations
        }
    }
    
    /**
     * method show()
     * Shows the page
     */
    public function show()
    {
        // check if the datagrid is already loaded
        if (!$this->loaded AND (!isset($_GET['method']) OR !(in_array($_GET['method'],  array('onReload', 'onSearch')))) )
        {
            if (func_num_args() > 0)
            {
                $this->onReload( func_get_arg(0) );
            }
            else
            {
                $this->onReload();
            }
        }
        parent::show();
    }
}
